%
% An idealized maize root system in a pot using parameters from literature 
% with 10% standard deviation.
%
% demonstrates the use of chemotropism
%
% See also: tropism, tropismObjective
%
%
% Copyright 2014 Daniel Leitner. See license.txt for details.
%

clear all;
set(0,'RecursionLimit',10000);

% Tap root 
p{1}.name = 'tap root';
p{1}.r = [2,0.2];
p{1}.lb = [0.7,0.07];          
p{1}.la = [15.7,1.57];                    
p{1}.ln = [0.7,0.07];                 
p{1}.nob = [50,5];  
p{1}.a = 0.03;   
p{1}.theta = 0;
p{1}.color = [1,0,0]; % red
%p{1}.tropism = [1,2,15*pi/180];  % gravitropism
p{1}.tropism = [3,2,15*pi/180];  % [type, N, sigma] Chemotropism: type=3, N is strength, sigma is stiffness
p{1}.dx= 0.25; 
p{1}.successor = [2,1];

% 1st order lateral
p{2}.name = '1st order lateral';
p{2}.r = [6.4, 0.64];
p{2}.lb = [0.7, 0.07];    
p{2}.la = [2.7, 0.27];                       
p{2}.ln = [0.7, 0.07];               
p{2}.nob = [7, 0.7];                  
p{2}.a = 0.02;                
p{2}.theta = [63*pi/180,0];      
p{2}.color = [140/255,125/255,40/255];
%p{2}.tropism = [1,0.5,40*pi/180];  % gravitropism
p{2}.tropism = [3,0.5,40*pi/180]; % [type, N, sigma] Chemotropism: type=3, N is strength, sigma is stiffness
p{2}.dx= 0.25;


% plant pot geometry 
r1=7;
r2=4;
h=15;
pot = @(p) distPot(p,r1,r2,h);

p = completeParameters(p,pot);

% Defines an initial concentration within the pot as a gradient from c01 to c02
% and stores it in a global variable
global C;
global X;
global Y;
global Z;

nx = 20; % resolution
ny = 20; 
nz = 20;
x = linspace(-r1,r1,nx); 
y = linspace(-r1,r1,ny); 
z = linspace(-h,0,nz);
[X,Y,Z] = meshgrid(x,y,z); % grid

c01 = 1e-4;   
c02 = 1e-3;   % gradient between c01 - c02

C = zeros(nx,ny,nz);
for i=1:nx
    u=(i-1)/(nx-1);
    C(i,:,:)=c02*u*ones(ny,nz) + (1-u)*c01*ones(ny,nz);
end

%
% Simulate
%
n0=7; % tap roots
r0=5; % cm
root = createRootSystem(0,0,2*rand(n0,1),zeros(7,1),r0); % initial letter
simtime = 40;% days

str = root;
for i = 1 : simtime

    disp(['Day ' num2str(i)]);
    
    str = applyRules(str,1); % simulates one day
   
    % 
    % insert code for nutrient movement in soil (altering global concentration C)
    %
    
end

hold on;

%
% plot root system
%
plotTubes(str);
X_ = linspace(-r1,r1,100); 
Y_ = linspace(-r1,r1,100);
Z_ = linspace(-h,0,40);
plotDistFunc(pot,X_,Y_,Z_,[0 0 1]); % approximate the pot boundaries
view(3);

%
% plot concentration
%
m=zeros(ny,nx,nz);
slice(X,Y,Z,C,0,0,-14.5);
axis equal;
view(90,30); 




