function writeRSML(filename, str, simtime, pos)
% writeRSML: writes a RSML file of the l-system string
%
% additionally saves the parameter set to the workspace
% 
%
% See also: convert2RSWMSinput
%
% Copyright 2014 Daniel Leitner. See license.txt for details.
%

global parameters;

if (nargin<4)
    pos.x=[0;0;0];
    pos.C=[0,0,0]; % black
    pos.a=0.1;
    pos.t=0;
    pos.R=eye(3);
end

%
% Define Meta Data
%
m.version = 5;
m.unit = 'cm';
m.resolution = 1;
m.last_DASH_modified=date;
m.software = 'RootBox';
%m.user = '';
%m.file_DASH_key = ''

im.label = filename;
m.image = im;
m.property_DASH_definitions.property_DASH_definition(1).label = 'length';
m.property_DASH_definitions.property_DASH_definition(1).type = 'float';
m.property_DASH_definitions.property_DASH_definition(1).unit = 'pixel';
%
m.property_DASH_definitions.property_DASH_definition(3).label = 'type';
m.property_DASH_definitions.property_DASH_definition(3).type = 'float';
m.property_DASH_definitions.property_DASH_definition(3).unit = '-';
%
m.property_DASH_definitions.property_DASH_definition(3).label = 'diameter';
m.property_DASH_definitions.property_DASH_definition(3).type = 'float';
m.property_DASH_definitions.property_DASH_definition(3).unit = 'cm';
%
m.property_DASH_definitions.property_DASH_definition(3).label = 'color';
m.property_DASH_definitions.property_DASH_definition(3).type = 'float';
m.property_DASH_definitions.property_DASH_definition(3).unit = 'rgb';
%
m.property_DASH_definitions.property_DASH_definition(3).label = 'parent_DASH_node';
m.property_DASH_definitions.property_DASH_definition(3).type = 'float';
m.property_DASH_definitions.property_DASH_definition(3).unit = '-';


%
% first convert to RSWMS (closer to RSML than the l-system string)
%
[seg, tip]=convert2RSWMSinput(str);

seg(1,:)= []; % remove artificial origin created by conver2RSWMSinput
seg(:,1)=seg(:,1)-1; % segment id
seg(:,5)=seg(:,5)-1; % connected id

[~,ix] = sort(seg(:,1)); % sort by segid
seg = seg(ix,:); 

[~,~,radius,color,time]=getSegments(str, pos); 
% the order is the same as in seg

% modify connected field (to which branch id instead of which segment id)
ind0 = find(seg(:,5)==0);
seg(ind0,5)=ind0; % if a segment has no previous segment its connected to itself
seg(:,5) = seg(seg(:,5),7); % insert branch ids of previous segment

%
% Define Roots
%
ind0 = find(tip(:,6)==1); % find roots with type 1
for i = 1 : length(ind0)
    root0(i) = getRoot(ind0(i), seg, tip, simtime, radius, color, time);
end
tree.scene.plant.root = root0;

%
% Write file
%
rsmlname = [filename, '.rsml'];
wPref.StructItem = false;
xml_write(rsmlname,tree,'rsml',wPref);

matname = [filename, '.mat']; % additionally save the global parameters
save(matname,'parameters');







function root = getRoot(id, seg, tip, simtime, radius,color,time)
% id        branchid
%

% converts a single root
seg_id = find(seg(:,7)==id); % seg(:,7) branchid
seg_cid = find(seg(:,5)==id); % which segments are connected to id
suc = unique(seg(seg_cid,7)); % successor branch ids
suc(suc==id)=[]; 

% fprintf('Branch %g, number of laterals: %g\n',id,length(suc));

% attribute
root.ATTRIBUTE.ID = id;

% geometry 
for j = 1 : size(seg_id,1)
    root.geometry.polyline.point(j).ATTRIBUTE.x = seg(seg_id(j),2);
    root.geometry.polyline.point(j).ATTRIBUTE.y = seg(seg_id(j),3);    
    root.geometry.polyline.point(j).ATTRIBUTE.z = seg(seg_id(j),4);    
end                                           

% properties
root.properties.length.ATTRIBUTE.value = tip(id,8);
root.properties.type.ATTRIBUTE.value = tip(id,6);
root.properties.diameter.ATTRIBUTE.value = 2*radius(seg_id(1));
root.properties.color.ATTRIBUTE.value = color(seg_id(1));

if ~isempty(suc)
    for j = 1 : length(suc) % parse successors        
        rootS(j) = getRoot(suc(j),seg, tip,simtime, radius,color,time);      
        rootS(j).properties.parent_DASH_node.ATTRIBUTE.value = id;            
    end    
    root.root = rootS;
else
    root.root = [];
end

% functions
root.functions.function.ATTRIBUTE.domain='polyline';
root.functions.function.ATTRIBUTE.name='age';
for j = 1 : size(seg_id,1)
    age = simtime-time(seg_id(j));
    if isnan(age) 
        age = 0;
    end
    root.functions.function.sample(j).ATTRIBUTE.value = age;     
end
